<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use \Doctrine\ORM\EntityManager;
use Exception;
use \Zeedhi\Framework\DTO\Response;
use \Setforms2\Shared\ORM\DateTime;

class Pla00160AjustarComensaisEstimadosCardapioController extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $servicoService;
	protected $crudService;
	protected $novoCodigo;
	protected $connection;
	protected $listas;
	protected $entityManager;
	protected $environment;

	public function __construct($servicoService, $listas, $entityManager, $environment, $novoCodigo, $crudService, $connection) {
		$this->service = $servicoService;
		$this->listas = $listas;
		$this->entityManager = $entityManager;
		$this->environment = $environment;
		$this->novoCodigo = $novoCodigo;
		$this->crudService = $crudService;
		$this->connection = $connection;
	}

    public function verificaNecessidadeJustificativa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $dtInicial = $row['PERIODO']['start'];
            $dtFinal = $row['PERIODO']['end'];
            $filiais = $row['TFILIAL'] === 'S' ? 'T' : $row['NMFILIAL'];
            $servicos = $row['TSERVICO'] === 'S' ? 'T' : $row['SERVICO'];
            $row['NRLISTA'] = $this->listas->retornaListaFilialServico(null, $filiais, 'T', $servicos, false, false, false);
            $diasCardBloq = $this->service->retornaDiasCardapioBloqueado($dtInicial, $dtFinal, $row['NRLISTA']);
            if($diasCardBloq[0]['EXISTE_CARD_BLOQUEADO'] > 0){
                $response->addDataSet(new DataSet('diasBloqueados', $diasCardBloq));
                return;
            }
            $justificativa = $this->service->verificaNecessidadeJustificativa();
            $response->addDataSet(new DataSet('justificativa', $justificativa));
        } catch (\Exception $e) {
            
        }
    }
    
    public function ajustarComensaisCardapio(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->entityManager->beginTransaction();
            $filiais = $row['TFILIAL'] === 'S' ? 'T' : $row['NMFILIAL'];
            $servicos = $row['TSERVICO'] === 'S' ? 'T' : $row['SERVICO'];
            $dtInicial = $row['PERIODO']['start'];
            $dtFinal = $row['PERIODO']['end'];
            $ajustePorcentagem = $row['AJUSTE_PORCENTAGEM'];
            $operacao = $row['OPERACAO'];
            $valorPorcentagem;
            $row['NRLISTA'] = $this->listas->retornaListaFilialServico(null, $filiais, 'T', $servicos, false, false, false);

            $cardapios = $this->service->retornaCardapios($dtInicial, $dtFinal, $row['NRLISTA']);

            if(!empty($cardapios)){
                foreach ($cardapios as $cards) {
                
                    if(!empty($cards['QTCLIESTICAR']) && $cards['QTCLIESTICAR'] != 0){
                                   
                        $novoValorQtcliesticar = $operacao == 'R' ? round($cards['QTCLIESTICAR'] * (1 - $ajustePorcentagem / 100)) : round($cards['QTCLIESTICAR'] * (1 + $ajustePorcentagem / 100));
                        $comensaisFuncionarios = $cards['QTFUNESTICAR'] ? $cards['QTFUNESTICAR'] : 0;
                        $comensaisDoacao = $cards['QTDOAESTICAR'] ? $cards['QTDOAESTICAR'] : 0;
                        $novoValorQtcomesticar = $novoValorQtcliesticar + $comensaisFuncionarios + $comensaisDoacao;
                        
                        $turnos = [
                            'QTESTITURNO1' => $cards['QTESTITURNO1'], 
                            'QTESTITURNO2' => $cards['QTESTITURNO2'], 
                            'QTESTITURNO3' => $cards['QTESTITURNO3'], 
                            'QTESTITURNO4' => $cards['QTESTITURNO4']
                        ];
                        $somaArrayTurnosOriginal = array_sum($turnos);
                        
                        foreach ($turnos as $index => $value) {
    
                            if(!empty($value) && $value != 0) {
                                $pesoPorcentagemTurno = ($value / $somaArrayTurnosOriginal) * 100;
                                $turnos[$index] = round(($novoValorQtcomesticar * $pesoPorcentagemTurno) / 100);
                            } else {
                                $turnos[$index] = $value;
                            }
                            
                        }
    
                        $diferenca = round($novoValorQtcomesticar - array_sum($turnos));

                        if ($diferenca != 0) {
                            for ($i = count($turnos) - 1; $i >= 0; $i--) {
                                $key = array_keys($turnos)[$i];
                                if ($turnos[$key] != 0 && $turnos[$key] !== null && $turnos[$key] + $diferenca >= 0 ) {
                                    $turnos[$key] += $diferenca;
                                    break;
                                }
                            }
                        }

                        $dadosUpdateCardadia = array(
                            "QTCOMESTICAR"   => $novoValorQtcomesticar,
                            "QTCLIESTICAR"   => $novoValorQtcliesticar,
                            "QTESTITURNO1"   => $turnos['QTESTITURNO1'],
                            "QTESTITURNO2"   => $turnos['QTESTITURNO2'],
                            "QTESTITURNO3"   => $turnos['QTESTITURNO3'],
                            "QTESTITURNO4"   => $turnos['QTESTITURNO4'],
                            "DTCARDAPIO"     => $cards['DTCARDAPIO'],
                            "CDFILIAL"       => $cards['CDFILIAL'],
                            "CDSERVFILI"     => $cards['CDSERVFILI'],
                            "NRCARDAPIO"     => $cards['NRCARDAPIO']
                        );

                        $this->service->updateCardadia($dadosUpdateCardadia);
                        $cardacli = $this->service->retornaCardacli($cards['CDFILIAL'], $cards['NRCARDAPIO']);
                        if(!empty($cardacli)){
                            $comensaisPontoEntrega = [];
                            foreach ($cardacli as $value) {
                                $comensaisPontoEntrega[$value['CDCLIENTE'] . '_' . $value['CDDESTFILI'] ] = $value['QTCOMESTICLI'];
                            }
                            
                            $somaArrayPontosOriginal = array_sum($comensaisPontoEntrega);
                            
                            foreach ($comensaisPontoEntrega as $index => $value) {
    
                                if(!empty($value) && $value != 0) {
                                    $pesoPorcentagemPonto = ($value / $somaArrayPontosOriginal) * 100;
                                    $comensaisPontoEntrega[$index] = round(($novoValorQtcliesticar * $pesoPorcentagemPonto) / 100);
                                } else {
                                    $comensaisPontoEntrega[$index] = $value;
                                }
                                
                            }
          
                            $diferencaPontoEntrega = round($novoValorQtcliesticar - array_sum($comensaisPontoEntrega));

                            if ($diferencaPontoEntrega != 0) {
                                for ($i = count($comensaisPontoEntrega) - 1; $i >= 0; $i--) {
                                    $key = array_keys($comensaisPontoEntrega)[$i];
                                    if ($comensaisPontoEntrega[$key] != 0 && $comensaisPontoEntrega[$key] !== null && $comensaisPontoEntrega[$key] + $diferencaPontoEntrega >= 0 ) {
                                        $comensaisPontoEntrega[$key] += $diferencaPontoEntrega;
                                        break;
                                    }
                                }
                            }
           
                            foreach ($cardacli as $cli) {
                                $dadosUpdateCardadia = array(
                                    "QTCOMESTICLI"   => $comensaisPontoEntrega[$cli['CDCLIENTE'] . '_' . $cli['CDDESTFILI']],
                                    "CDFILIAL"   => $cli['CDFILIAL'],
                                    "NRCARDAPIO"   => $cli['NRCARDAPIO'],
                                    "CDCLIENTE"   => $cli['CDCLIENTE'],
                                    "CDDESTFILI"   => $cli['CDDESTFILI']
                                );
                               $this->service->updateCardacli($dadosUpdateCardadia);
                            }
                        }
                    }
                    if (isset($row['JUSTIFICATIVA'])) {

                        $data_hora = new DateTime();
                        $data_hora_formatada = $data_hora->format('d/m/Y H:i:s');
                        $nrorg = $this->environment->getNrOrg();
                        $nrjustaltercomens = $this->novoCodigo->getCodeByTableName('JUSTALTERCOMENS', $nrorg);
                        $dadosInsert = array(
                            "NRJUSTALTERCOMENS" => $nrjustaltercomens,
                            "CDFILIAL" => $cards['CDFILIAL'],
                            "CDSERVFILI" => $cards['CDSERVFILI'],
                            "DTCARDAPIO" => $cards['DTCARDAPIO'],
                            "QTCOMENSTOTOLD" => $cards['QTCOMESTICAR'],
                            "QTCOMENSTOTNEW" => $novoValorQtcomesticar,
                            "DTHRALTERCOMENS" => $data_hora_formatada,
                            "CDOPERADOR" => $this->environment->getCdoperador(),
                            "DSJUSTIFICATIVA" => $row['JUSTIFICATIVA'],
                            "DSACAO" => $row['ACAO']
                        );
                        $this->service->insertJustificativa($dadosInsert);
                    }
                }
            }
            $this->entityManager->commit();
            $response->addDataSet(new DataSet('ajusteComSucesso'));
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

}